##############################################################################
#    Copyright (C) 2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, zLOG, random, string
from Globals import InitializeClass, DTMLFile, package_home
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem
from Products.PageTemplates import PageTemplateFile
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore import CMFCorePermissions
from DateTime import DateTime

_zptdir = os.path.join(package_home(globals()), 'www')

class MediaManagerException( Exception ):
    pass

class NewsTool( UniqueObject, SimpleItem ):

    """
    A tool for displaying news :-)
    """

    id = 'portal_newstool'
    meta_type = 'News Tool'

    security = ClassSecurityInfo()
    
    manage_options = ( { 'label' : 'Overview', 'action' : 'manage_overview' }
                     , { 'label' : 'Subjects', 'action' : 'manage_subjects' }
                     ) + SimpleItem.manage_options

    #
    #   ZMI methods
    #
    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_overview' )
    manage_overview = PageTemplateFile.PageTemplateFile( 'explainNewsTool', _zptdir)

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_subjects')
    manage_subjects = PageTemplateFile.PageTemplateFile('setNewsSubjects', _zptdir)

    #
    #   'portal_newstool' interface methods
    #

                
    security.declarePublic('getNewsSubjects')
    def getNewsSubjects(self):
        """
        Returns the subjects for the news box
        """
        if hasattr(self, 'news_subjects'):
            zLOG.LOG('News Tool', zLOG.INFO, 'Getting these news subjects: %s' % str(self.news_subjects))
            return string.join(self.news_subjects, '\n')
        else:
            zLOG.LOG('News Tool', zLOG.INFO, 'Getting no news subjects')
            return None
            
    security.declarePublic('setNewsSubjects')
    def setNewsSubjects(self, subjects):
        """
        Sets the subjects for the news box
        """
        zLOG.LOG('News Tool', zLOG.INFO, 'Setting these subjects: %s' % str(subjects))
        self.news_subjects = subjects

    security.declarePublic('setNewsTypes')
    def setNewsTypes(self, types):
        """
            Set which portal types should be displayed in the news box.
        """
        self.news_types = types
                
    security.declarePublic('getNewsTypes')
    def getNewsTypes(self):
        """
            Return the list of portal types that should be displayed in the news box
        """
        if hasattr(self, 'news_types'):
            typelist = self.news_types
        else:
            typelist = None
        return typelist
        
    security.declarePublic('setListCount')
    def setListCount(self, listcount):
        """
        Sets the number of items to show up in the news list
        """
        self.list_count = listcount
        
    security.declarePublic('getListCount')
    def getListCount(self):
        """
        Returns the number of items to show up in the news list
        """
        if hasattr(self, 'list_count'):
            return int(self.list_count)
        else:
            return 10

    security.declarePublic('queryCatalog')
    def queryCatalog(self):
        """
            Returns a list of 'brains' objects from the portal catalog
            based on the portal types selected.
        """
        catalog = getToolByName(self, 'portal_catalog')
        
        if hasattr(self, 'news_subjects'):
            subj = self.news_subjects
        else:
            subj = None
        types = self.getNewsTypes()
        result = None
        if types:
            if subj:
                result = catalog.searchResults(Type=types
                                             , Subject=subj
                                             , sort_on='Date'
                                             , sort_order='reverse')
            else:
                result = catalog.searchResults(Type=types
                                             , sort_on='Date'
                                             , sort_order='reverse')
        
        format = [{'name':'Icon', 'type':'icon'}
                , {'name':'Title', 'type':'title'}
                , {'name':'Description', 'type':'description'}
                , {'name':'Date', 'type':'date'}]
        items = []
        if result:
            for entry in result:
                object = entry.getObject()
                if object:
                    items.append(
                        {'Icon':{'src': object.getIcon()}
                       , 'Title':{'href': object.absolute_url(), 'title': object.title_or_id}
                       , 'Description':{'text':object.description}
                       , 'Date':{'text':DateTime(object.ModificationDate()).strftime('%d/%m-%Y %H:%M')}}
                       )
            return {'format':format, 'entries': items}
        else:
            return None

InitializeClass( NewsTool )
